// butpwm1.c - PWM with duty cycle ramped under control by buttons
// PWM at 128Hz, 256 counts per cycle from ACLK, starting at 50%
// Buttons polled once per cycle to inc or dec pulse width by 1
// All action in TACCR0 CCIFG interrupt, fast MCLK to make time
// Olimex 1121STK, LED1 on P2.3 = TA1, active low; piezo off;
//   down button B1 on P2.1, up button B2 on P1.2, both active low
// J H Davies, 2007-07-19; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device
#include <intrinsics.h>			// Intrinsic functions
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	BCSCTL1 = XT2OFF | RSEL2 | RSEL1 | RSEL0;	// Highest DCO range
	DCOCTL = DCO2 | DCO1 | DCO0;	// Highest DCO step, no modulation
// Set up Timer_A for PWM on active low LED, channel 1
	TACCR0 = 0x00FF;			// 128Hz from 16KHz timer clock
	TACCR1 = 0x0080;			// Duty cycle D = 0.5 initially
	TACCTL0 = CCIE;				// Enable interrupts on compare
	TACCTL1 = OUTMOD_3;			// Set/reset mode for negative PWM
// Configure ports 1 and 2; redirect P2.3 to Timer_A
	P1OUT = BIT0 | BIT1;		// Output high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3;				// Route P2.3 to TA1
	P2OUT = BIT3 | BIT4;		// LEDs off (active low); piezo off
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
// Start timer from ACLK, no division, Up mode, clear, no interrupts
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// Enter LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// ISR for CCIFG0, safest place to update PWM
// Reads buttons and updates TACCR1; "down" button on P2.1 has priority
// Further CCIFG0 ints disabled and ports re-enabled if both buttons up
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)		// Flag cleared automatically
{
	if (P2IN_bit.P2IN_1 == 0) {			// Down button pressed?
		if (TACCR1 > 0) {				// Yes: Above 0 limit?
			--TACCR1;					// Yes: decrease duty cycle
		}
	} else if (P1IN_bit.P1IN_2 == 0) {	// Up button pressed?
		if (TACCR1 <= TACCR0) {			// Yes: below 100% limit?
			++TACCR1;					// Yes: increase duty cycle
		}
	}
}
