// morsetim1.c - sends dots and dashes in message[] on LED and piezo
// Timer clock is ACLK, up mode, period 32 for 1ms
// Drive piezo from this in ISR, count ticks for longer delays
// LPM3 between interrupts, wake main routine every 100ms
// Olimex 1121STK, LED1 active low on P2.3,
//   piezo between P2.0 and P2.5
// J H Davies, 2006-10-18; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>					// Specific device, newer format
#include <intrinsics.h>					// Intrinsic functions
//----------------------------------------------------------------------
// Pins for LEDs and piezo, all on port 2
#define	LED1	BIT3
#define	PIEZO1	BIT0
#define	PIEZO2	BIT5
// Multiplier for 0.1s delay based on timer period of 1ms
#define TENTHLOOPS	100
// Durations of symbols in morse code in units of 0.1s
//   DOT and DASH are symbols sent, SPACE separates symbols,
//   LETTER separates letters; ENDTX terminates message
#define DOT		2
#define DASH	6
#define LETTER	0
#define SPACE	2
#define ENDTX	0xFF
//----------------------------------------------------------------------
const unsigned char message[] =
{DOT,DOT,DOT,LETTER,DASH,DASH,DASH,LETTER,DOT,DOT,DOT,ENDTX};	// SOS
//----------------------------------------------------------------------
void main (void)
{
	unsigned int i;					// Counter to step through message
	unsigned int tenths;			// Counter for duration in 0.1s

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	P2OUT = PIEZO1;					// Preload LED1 on, piezo ready
	TACCR0 = 32;					// Upper limit of count for TAR
	TACCTL0 = CCIE;					// Enable interrupts on Compare 0
	TACTL = MC_1|TASSEL_1|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer

	for (i = 0; message[i] != ENDTX; ++i) {	// Message loop
		if (message[i] > 0) {			// Nonzero duration to send?
			P2DIR = LED1|PIEZO1|PIEZO2;		// LED1 and piezo on
			for (tenths = 0; tenths < message[i]; ++tenths) {
				__low_power_mode_3();		// LPM3; returns after 100ms
			}					// Timer A sounds piezo and counts ticks
		}
		P2DIR = 0;					// LED1 and piezo off
		for (tenths = 0; tenths < SPACE; ++tenths) {	// Send space
			__low_power_mode_3();	// LPM3; returns after 100ms
		}
	}
	for (;;) {						// Infinite loop, trap processor
		__low_power_mode_4();		// LPM4 (RAM retention only)
	}								// Should never be reached
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Processor usually returns to LPM3 after ISR
//   but goes to active mode when ticks reaches 0
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
	static unsigned int ticks = TENTHLOOPS;	// Counts to give 100ms

	P2OUT ^= (PIEZO1|PIEZO2);			// Toggle piezo
	if (--ticks == 0) {					// Ticks timed out (100ms)?
		ticks = TENTHLOOPS;				// Reload counter
		__low_power_mode_off_on_exit();	// Active mode on exit
	}
}
