// quiz2.c - Light LED for 1s corresponding to first button pressed
// Sounds buzzer while LED lit
// Interrupts on input high->low edges, LPM4 while waiting
// Timer_A drives piezo and counts 1s delay, LPM3 while waiting
// Most action in main routine, minimum in ISRs
// Olimex 1121STK board, LED1,2 active low on P2.3,4
//   buttons B1, B2 active low on P2.1, P1.2
// J H Davies, 2006-12-19; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Specific device, new format
#include <intrinsics.h>				// Intrinsic functions
//----------------------------------------------------------------------
// Pins for LED, button and piezo sounder (port 2)
#define	LED1	BIT3
#define	LED2	BIT4
#define	B1		BIT1
#define	B2		BIT2
#define	PIEZO1	BIT0
#define	PIEZO2	BIT5
// This program uses bit masks rather than fields to show the difference
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;			// Stop watchdog timer
	P2OUT = PIEZO1|LED1|LED2;		// LEDs off; set up piezo
	P2DIR = PIEZO1|PIEZO2|LED1|LED2;	// Output pins for LEDs and piezo
	P2IES = B1;						// Select high-to-low triggers
	P1IES = B2;						//   for P2.1 and P1.2
	TACCR0 = 32;					// Limit of count for TAR (10ms)
	TACCTL0 = CCIE;					// Enable interrupts on Compare 0
	for (;;) {						// Loop forever
// Be careful with precedence of operators in the next expression!
		while ((P2IN & B1) == 0 || (P1IN & B2) == 0) {
		}				// Wait for buttons to be released (active low)
		do {
			P1IFG = 0;				// Clear any pending interrupts...
			P2IFG = 0;				// ...until none remain
		} while ((P1IFG != 0) || (P2IFG != 0));
		P2IE = B1;					// Enable interrupts on edge
		P1IE = B2;
		__low_power_mode_4();		// LPM4, all clocks stopped
// Wait for button to be pressed
		P2IE = 0;					// Disable interrupts on edge
		P1IE = 0;
		TACTL = MC_1|TASSEL_1|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
		__low_power_mode_3();		// LPM3, ACLK running for timer
// Wait for timer to expire
		TACTL = 0;					// Disable timer (mode 0)
		P2OUT |= (LED1|LED2);		// Both LEDs off (active low!)
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for P2.1 (button 1)
// Acknowledge interrupt, light LED, return to active mode on exit
//----------------------------------------------------------------------
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)
{
	P2IFG = 0;						// Clear interrupt flag(s)
	P2OUT &= ~LED1;					// Light corresponding LED
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
//----------------------------------------------------------------------
// Interrupt service routine for P1.2 (button 2)
// Acknowledge interrupt, light LED, return to active mode on exit
// Should really check which pin caused interrupt but unnecessary here
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	P1IFG = 0;						// Clear interrupt flag(s)
	P2OUT &= ~LED2;					// Light corresponding LED
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Toggles piezo sounder on each call; usually returns to LPM3 after ISR
//   but goes to active mode when ticks reaches 0 after 1s
//----------------------------------------------------------------------
#define SECONDLOOPS	1000
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
	static unsigned int ticks = SECONDLOOPS;	// Counts to give 1s

	P2OUT ^= PIEZO1|PIEZO2;				// Toggle piezo
	if (--ticks == 0) {					// Ticks timed out (1s)?
		ticks = SECONDLOOPS;			// Re-initialize counter
		__low_power_mode_off_on_exit();	// Active mode on exit
	}
}
