// react4.c - Reaction timer using Timer_A, display in ms
// Press S2, wait for LED4, press S1 as quickly as possible!
// Timer_A used for random delay and reaction time
// As react2.c with detection of overflow and deep sleep on inactivity
// Interrupts from TAIFG used to detect overflow and inactivity
// TI Experimenter's Board with F4619, default clocks
// J H Davies, 2007-06-12; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils.h"				// SBLCDA4 utility functions
#define	TIMEOUT	2					// Number of 2s delays before LPM4
#define	LED		P5OUT_bit.P5OUT_1	// Pin for LED
#define	START_BUTTON	P1IN_bit.P1IN_1	// Button S2 to start trial
void PortsInit (void);				// Set up i/o ports
//----------------------------------------------------------------------
void main (void)
{
	uint32_t	ReactionTime;		// Computed reaction time, 32 bits
	uint8_t		WaitLoops;			// Count loops before entering LPM4

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// Configure load caps (14pF)
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
// Capture falling edge of CCI0A, synchronized, no ints yet
	TACCTL0 = CM_2 | CCIS_0 | SCS | CAP;
// Start timer: ACLK, no prescale, continuous mode, interrupts, clear
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR | TAIE;
	for (;;) {						// Loop forever
		P1IFG = 0;					// Clear any pending interrupts
		P1IE = BIT1;				// Enable interrupts on P1.1 = B2
// Loop until timeout while waiting for user to press start button
		for (WaitLoops = TIMEOUT; START_BUTTON == 1 && WaitLoops > 0;
				--WaitLoops) {
			__low_power_mode_3();	// Wait for button to be pressed
		}
// Did we leave loop because of timeout or start button pressed?
		if (START_BUTTON == 1) {	// Button still up, timed out
			LCDACTL_bit.LCDON = 0;	// Disable LCD before ACLK stops
// Enter LPM4 until button pressed; ACLK stops, no TAIFG ints possible
			__low_power_mode_4();	// LPM4 until button pressed
			LCDACTL_bit.LCDON = 1;	// Restart LCD
			DisplayLine();			// ----- on LCD to show readiness
		} else {					// Button pressed, start trial
			P1IE = 0;				// Disable further P1 interrupts
			TACTL_bit.TAMC = 0;		// Hold timer during test and adjust
			if (TAR > 0x8000) {		// Will delay be < 1s?
				TAR -= 0x8000;		// Yes: add 1s so 1s < delay < 2s
			}
			TACTL |= MC_2;			// Restart timer
			__low_power_mode_3();	// Wait for random delay
			LED = 1;				// Turn on LED to start trial
			TACCTL0_bit.CCIFG = 0;	// Clear any pending interrupt
			TACCTL0_bit.CCIE = 1;	// Enable TACCR0 capture interrupts
			__low_power_mode_3();	// Wait for user to press button
			TACCTL0_bit.CCIE = 0;	// Disable further capture ints
			LED = 0;				// Turn off LED
// Interrupt from TACCR0 if button pressed, from TAIFG if overflowed
// Read state of button from capture/compare input CCI of TACCR0
			if (TACCTL0_bit.CCI == 0) {	// Button pressed, compute time
				ReactionTime = TACCR0;	// Starting time was 0 counts
				ReactionTime *= 1000;	// Convert seconds to ms
				ReactionTime >>= 15;	// Economical /= 32K (f_ACLK)
				DisplayUint ((uint16_t) ReactionTime);	// Display time
			} else {				// Trial overflowed (no press in 2s)
				DisplayErr();		// Display "Error" on LCD
			}
		}
	}								// Finished! - go and start again
}
//----------------------------------------------------------------------
// Interrupt service routine for port 1; need to clear flag
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)	// Flag NOT cleared automatically!
{
	P1IFG = 0;						// Acknowledge interrupt
	__low_power_mode_off_on_exit();
}
//----------------------------------------------------------------------
// Interrupt service routine for TAIFG, called after random interval
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Flag NOT cleared automatically!
{
	TACTL_bit.TAIFG = 0;			// Acknowledge interrupt
	__low_power_mode_off_on_exit();
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG, called on capture
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Flag cleared automatically
{
	__low_power_mode_off_on_exit();
}
//----------------------------------------------------------------------
// Initialize ports for board: pins are outputs driven low by default
// Many of these will be overwritten by LCD initialization later
//----------------------------------------------------------------------
void PortsInit (void)
{
// Port 1: 0 = SW1, 1 = SW2, others used for Chipcon (not placed)
	P1OUT = 0;
	P1DIR = 0xFF & ~(BIT0|BIT1);		// P1.0,1 input, others output
	P1SEL = BIT0;						// P1.0 = S1 to Timer_A CCI0A
	P1IES = BIT1;						// Interrupts on falling edge
// Port 2: 1 = LED2, 2 = LED1, 3 = mic power, 4 = UCA0TXD, 5 = UCA0RXD
	P2OUT = 0;
	P2DIR = 0xFF & ~BIT5;				// P2.5 must be input
// Port 3: 0-3 = comms with F2013, 5 = buzzer (active low)
//	P3OUT = BIT1 | BIT2;				// F2013 might tie these high
//	P3DIR = 0xFF & ~BIT5;				// P3.5 must be high
	P3OUT = BIT1 | BIT2 | BIT5;			// F2013 might tie 1,2 high;
	P3DIR = 0xFF;						//   P3.5 high for buzzer off
// Port 4: 2-5 used for Chipcon (not placed)
	P4OUT = 0;
	P4DIR = 0xFF;
// Port 5: 1 = LED4, 2-4 = COM1-3 for LCD
	P5OUT = 0;
	P5DIR = 0xFF;
// Port 6: All except 6 used for opamp external components
	P6OUT = 0;
	P6DIR = 0xFF;
// Port 7: 4-7 = LCD segments
	P7OUT = 0;
	P7DIR = 0xFF;
// Port 8: LCD segments
	P8OUT = 0;
	P8DIR = 0xFF;
// Port 9: LCD segments
	P9OUT = 0;
	P9DIR = 0xFF;
// Port 10: 0-5 = LCD segments
	P10OUT = 0;
	P10DIR = 0xFF;
}
