// rtclock4.c - simple clock using SBLCD4 on TI Experimenter's Board
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Flashes colon every 0.5s, updates display every minute
// Displays date when button S1 on 1,0 held down
// J H Davies, 2007-05-13; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
	uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
// LCD segment definitions (SoftBaugh SBLCDA4)
#define SEG_A   BIT0					//  AAAA
#define SEG_B   BIT1					// F    B
#define SEG_C   BIT2					// F    B
#define SEG_D   BIT3					//  GGGG
#define SEG_E   BIT6					// E    C
#define SEG_F   BIT4					// E    C
#define SEG_G   BIT5					//  DDDD
#define SEG_H   BIT7					// colon, point etc
// Patterns for hexadecimal characters
const uint8_t LCDHexChar[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,			// '0'
	SEG_B | SEG_C,											// '1'
	SEG_A | SEG_B | SEG_D | SEG_E | SEG_G,					// '2'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_G,					// '3'
	SEG_B | SEG_C | SEG_F | SEG_G,							// '4'
	SEG_A | SEG_C | SEG_D | SEG_F | SEG_G,					// '5'
	SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,			// '6'
	SEG_A | SEG_B | SEG_C,									// '7'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// '8'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,			// '9'
	SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G,			// 'A'
	SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,					// 'b'
	SEG_A | SEG_D | SEG_E | SEG_F,							// 'C'
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,					// 'd'
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,					// 'E'
	SEG_A | SEG_E | SEG_F | SEG_G,							// 'F'
};
// Segments of "digit 10", half-digit on right plus special symbols
#define	MEMSEG	BIT7			// 'M' on character
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
void InitRTC (void);
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;			// Stop watchdog timer
	P1IES = BIT0;						// High-to-low transition
	P1IE = BIT0;						// Enable interrupts on P1.0
	InitLCD();							// Initialize SBLCDA4
	InitRTC();							// Initialize and start RTC
	for (;;) {							// Loop forever with interrupts
		__low_power_mode_3();			// ACLK with interrupts
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memories used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4|BIT3|BIT2;				// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Initialize and start basic timer and real-time clock (ACLK = 32KHz)
//----------------------------------------------------------------------
void InitRTC (void)
{
// BCD mode, stop RTC, calendar mode (clock from BTCNT2.6), 1min flag
//   (interrupt remains under control of basic timer, not RTC)
// Clears RTC registers and configures basic timer appropriately
	RTCCTL = RTCBCD | RTCHOLD | RTCMODE_3 | RTCTEV_0;
// Initialize time and date
	RTCTIM0 = 0x5900;					// Minutes and seconds
	RTCHOUR = 0x17;						// After-tea relaxation
	RTCDOW = 0;							// Day of week - Sunday = 0?
	RTCDATE = 0x0513;					// Month and day
	RTCYEAR = 0x2007;					// Year (4 digits)
// Set basic timer for 0.5s interrupts (RTC overrides SSEL, HOLD, DIV)
	BTCTL = BT_fCLK2_DIV64;				// Divide frequency by 64
	BTCNT1 = 0;							// Clear counters
	BTCNT2 = 0;
	IE2_bit.BTIE = 1;					// Enable basic timer interrupts
	RTCCTL &= ~RTCHOLD;					// Start RTC - No bitfield!
	RTCCTL |= RTCFG;					// Set flag and...
	IFG2_bit.BTIFG = 1;					// ...request an interrupt to
}										//   start display
//----------------------------------------------------------------------
// Interrupt service routine for basic timer; flag cleared automatically
// Time stored in RTC as seconds, minutes and hours in BCD format
// WARNING: potential problem of accessing RTC registers asynchronously
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)
{
	LCDMem[2] ^= SEG_H;					// Toggle colon separator
	if ((RTCCTL & RTCFG) != 0) {		// Time needs to be updated?
		RTCCTL &= ~RTCFG;				// Clear flag manually
		LCDMem[1] = LCDHexChar[RTCMIN & 0x0F];
		LCDMem[2] = LCDHexChar[(RTCMIN >> 4) & 0x0F];
		LCDMem[3] = LCDHexChar[RTCHOUR & 0x0F];
		LCDMem[4] = LCDHexChar[(RTCHOUR >> 4) & 0x0F];
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for pushbutton S1 on P1.0, active low
// Display date if button pressed and disable update of display on time
// Revert to updating display for time if button released
// Lazy coding - I assume that no other port 1 interrupts are used!
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	P1IFG = 0;							// Clear all - no others used
	if (P1IES != 0) {					// Button pressed
		P1IES = 0;						// Wait for release next
		IE2_bit.BTIE = 0;				// Disable basic timer ints
		LCDMem[0] = LCDHexChar[0x0D];	// Show 'd' for day
		LCDMem[1] = LCDHexChar[RTCDAY & 0x0F];
		LCDMem[2] = LCDHexChar[(RTCDAY >> 4) & 0x0F];
		LCDMem[3] = 0;					// Gap between month and day
		LCDMem[4] = 0;					// Gap between month and day
		LCDMem[5] = LCDHexChar[RTCMON & 0x0F];
		LCDMem[6] = LCDHexChar[(RTCMON >> 4) & 0x0F];
		LCDMem[10] |= MEMSEG;			// Show 'M' for month
	} else {							// Button released
		P1IES = BIT0;					// Wait for press next
		LCDMem[0] = 0;					// Clear digits used
		LCDMem[1] = 0;
		LCDMem[2] = 0;
		LCDMem[5] = 0;
		LCDMem[6] = 0;
		LCDMem[10] &= ~MEMSEG;			// Clear 'M'
		IE2_bit.BTIE = 1;				// Re-enable basic timer ints
		RTCCTL |= RTCFG;				// Set minute flag and...
		IFG2_bit.BTIFG = 1;				// ...raise an interrupt to
	}									//   display time
}
