// sclock3.c - simple clock using SBLCD4 on TI Experimenter's Board
// Displays hh:mmA/P, 12-hour format, flashes colon with period of 1s
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Timer clock 32KHz ACLK, no division, up mode, period 0x4000 = 1/2s
// J H Davies, 2006-12-02; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
// LCD segment definitions (SoftBaugh SBLCDA4)
#define SEG_A   BIT0					//  AAAA
#define SEG_B   BIT1					// F    B
#define SEG_C   BIT2					// F    B
#define SEG_D   BIT3					//  GGGG
#define SEG_E   BIT6					// E    C
#define SEG_F   BIT4					// E    C
#define SEG_G   BIT5					//  DDDD
#define SEG_H   BIT7					// colon, point etc
// Patterns for hexadecimal characters
const uint8_t LCDHexChar[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,			// "0"
	SEG_B | SEG_C,											// "1"
	SEG_A | SEG_B | SEG_D | SEG_E | SEG_G,					// "2"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_G,					// "3"
	SEG_B | SEG_C | SEG_F | SEG_G,							// "4"
	SEG_A | SEG_C | SEG_D | SEG_F | SEG_G,					// "5"
	SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,			// "6"
	SEG_A | SEG_B | SEG_C,									// "7"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// "8"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,			// "9"
	SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G,			// "A"
	SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,					// "b"
	SEG_A | SEG_D | SEG_E | SEG_F,							// "C"
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,					// "d"
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,					// "E"
	SEG_A | SEG_E | SEG_F | SEG_G,							// "F"
};
// Patterns for AM ("A"), PM ("P") and blank
const uint8_t LCDAMChar = SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G;
const uint8_t LCDPMChar = SEG_A | SEG_B | SEG_E | SEG_F | SEG_G;
const uint8_t LCDBlankChar = 0;
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;				// Stop watchdog timer
	InitLCD();							// Initialize SBLCDA4
	TACCR0 = 0x4000;					// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable Compare 0 interrupts
	TACTL = MC_1|TASSEL_1|TACLR;		// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	for (;;) {							// Loop forever
		__low_power_mode_3();			// ACLK continues to run
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memory used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4|BIT3|BIT2;				// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Time stored as mins and hours in BCD format with 12-hour format
//   changes am/pm after 11:59; 12:59 -> 1:00 (no leading zero)
//----------------------------------------------------------------------
#define MAXHALFSECS	119
#define	MAXMINS		0x59
#define	MAXHOURS	0x11
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)
{
	static unsigned short halfsecs = MAXHALFSECS;	// Don't need BCD
	static unsigned short minutes = MAXMINS;	// initial time in BCD
	static unsigned short hours = MAXHOURS;
	static enum {forenoon, afternoon} halfday = afternoon;	// am/pm

	LCDMem[2] ^= SEG_H;						// Toggle colon separator
	if (halfsecs < MAXHALFSECS) {
		++halfsecs;
	} else {
		halfsecs = 0;
		if (minutes < MAXMINS) {
			minutes = __bcd_add_short(minutes, 0x01);
		} else {
			minutes = 0x00;
			if (hours < MAXHOURS) {				// Simple increment
				hours = __bcd_add_short(hours, 0x01);
			} else if (hours == MAXHOURS) {		// 11:59, swap am<->pm
				hours = __bcd_add_short(hours, 0x01);
				if (halfday == afternoon) {
					halfday = forenoon;
					LCDMem[0] = LCDAMChar;
				} else {
					halfday = afternoon;
					LCDMem[0] = LCDPMChar;
				}
			} else {							// 12:59, 01:00 next
				hours = 0x01;
			}									// Update display
			if (hours < 0x10) {					// Leading zero?
				LCDMem[4] = LCDBlankChar;		// Yes: suppress
			} else {
				LCDMem[4] = LCDHexChar[(hours >> 4) & 0x0F];
			}
			LCDMem[3] = LCDHexChar[hours & 0x0F];
		}										// Update display
		LCDMem[2] = LCDHexChar[(minutes >> 4) & 0x0F];
		LCDMem[1] = LCDHexChar[minutes & 0x0F];
	}
}
