// sd16bat2.c - check SD16A internal supply voltage
// "Single" (actually 4) conversion of SD16A channel A5
// Home-made analog demo board for MSP430F20xx:
//   100n cap on P1.3 for Vref, LED active high on P2.6
// Calibrated 1MHz DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-10-03; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED
// Threshold for 3.0V with FSD = 0.6V and 16-bit range (unipolar)
// Internal voltage divided by 11: threshold = 2^16 * 3.0 / (11 * 0.6)
#define	BATTERY_THRESHOLD	((0x10000*30)/(11*6))	// = 29789 = 0x745D

void main (void)
{
	volatile uint16_t i;			// Loop counter for startup delay

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = 0;
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
	P1SEL = BIT3;					// Enable Vref output to capacitor
// Configure SD16A: clock from SMCLK, no division, internal reference on
	SD16CTL = SD16XDIV_0 | SD16DIV_0 | SD16SSEL_1 | SD16REFON;
// Unipolar output, single convs, OSR = 256, interrupts on finish
	SD16CCTL0 = SD16SNGL | SD16UNI | SD16OSR_256 | SD16IE;
// PGA gain = 1, supply volt divider A5+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_5 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
	for (i = 0xFFFF; i > 0; --i) {	// Delay to allow Vref to stabilize
	}
	for (;;) {						// Loop forever
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion
		__low_power_mode_0();
		if (SD16MEM0 < BATTERY_THRESHOLD) {	// Is supply voltage low?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
//----------------------------------------------------------------------
// ISR for SD16A: clear flag and return to main routine in active mode
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// NOT acknowledged automatically
{
	SD16CCTL0_bit.SD16IFG = 0;		// Clear flag, acknowledge interrupt
	__low_power_mode_off_on_exit();	// Return to active mode
}
