// testcp2.c - test charge pump for display on TI Experimenter's Board
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Press button S2 (P1.1) to increase voltage for charge pump,
//   press S1 (P1.0) to decrease
// Current voltage displayed in mV, updated in ISR for P1IFG
// J H Davies, 2007-05-07; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
//----------------------------------------------------------------------
#define	S1		P1IFG_bit.P1IFG_0	// Interrupt flags for switches
#define	S2		P1IFG_bit.P1IFG_1
#define LCDMEMS	11					// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
// LCD segment definitions (SoftBaugh SBLCDA4)
#define SEG_A   BIT0					//  AAAA
#define SEG_B   BIT1					// F    B
#define SEG_C   BIT2					// F    B
#define SEG_D   BIT3					//  GGGG
#define SEG_E   BIT6					// E    C
#define SEG_F   BIT4					// E    C
#define SEG_G   BIT5					//  DDDD
#define SEG_H   BIT7					// colon, point etc
// Patterns for hexadecimal characters
const uint8_t LCDHexChar[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,			// "0"
	SEG_B | SEG_C,											// "1"
	SEG_A | SEG_B | SEG_D | SEG_E | SEG_G,					// "2"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_G,					// "3"
	SEG_B | SEG_C | SEG_F | SEG_G,							// "4"
	SEG_A | SEG_C | SEG_D | SEG_F | SEG_G,					// "5"
	SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,			// "6"
	SEG_A | SEG_B | SEG_C,									// "7"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// "8"
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,			// "9"
	SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G,			// "A"
	SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,					// "b"
	SEG_A | SEG_D | SEG_E | SEG_F,							// "C"
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,					// "d"
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,					// "E"
	SEG_A | SEG_E | SEG_F | SEG_G,							// "F"
};
// More useful patterns
const uint8_t LCDMinusChar = SEG_G;
const uint8_t LCDBlankChar = 0;
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
void DisplayInt (int16_t IntValue);
extern uint32_t UIntToBCD (uint16_t UIntValue);		// assembly language
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;			// Stop watchdog timer
	P1IE = BIT0 | BIT1;				// Enable interrupts on edge
	P1IES = BIT0 | BIT1;			// Sensitive to negative edge (H->L)
	P1IFG = 0;						// Clear any pending interrupts
	InitLCD();						// Initialize SBLCDA4
	DisplayInt(0);					// Charge pump initially has V = 0
	for (;;) {						// Loop forever
		__low_power_mode_3();		// Leave ACLK running for LCD
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {	// Clear LCD memory used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4 | BIT3 | BIT2;		// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = LCDCPEN;		// Charge pump enabled
	LCDAVCTL1 = 0;				// Zero voltage output from pump
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Display signed, 16-bit integer (int16_t)
// Strip sign, convert unsigned value to BCD and display
// Leading zeros suppressed; BCD value does not exceed 5 digits
//----------------------------------------------------------------------
#define	LCDDIGITS	7					// Number of digits in display
void DisplayInt (int16_t IntValue)
{
	uint8_t i;							// Index for LCD array
	uint32_t BCDValue;					// Value converted bin to BCD
	enum {plus, minus} sign;

	if (IntValue >= 0) {				// Keep track of sign
		sign = plus;
	} else {
		sign = minus;
		IntValue = -IntValue;			// Conversion needs IntValue>=0
	}
	BCDValue = UIntToBCD (IntValue);	// Convert binary to BCD
	i = 0;								// Index for LCD memories
	do {								// Store pattern for next digit
		LCDMem[i++] = LCDHexChar[BCDValue & 0x000F];
		BCDValue >>= 4;					// Move next nibble down
	} while (BCDValue > 0);				// (Always display first digit)
	if (sign == minus) {
		LCDMem[i++] = LCDMinusChar;		// Prepend minus sign
	}
	while (i < LCDDIGITS) {				// Clear more significant digits
		LCDMem[i++] = LCDBlankChar;		//   of numerical display
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for Port 1 input
// Only one bit is active so no need to check which
//----------------------------------------------------------------------
// Table of LCD voltages in mV
const int16_t voltages[] = {0000, 2600, 2660, 2720, 2780, 2840,
	2900, 2960, 3020, 3080, 3140, 3200, 3260, 3320, 3380, 3440};
#define NVOLTAGES	(sizeof(voltages)/sizeof(int16_t))
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
	static uint8_t index = 0;			// Current index into table

	if (S1 == 1) {						// S1 pressed, decrease voltage
		if (index > 0) {				// Decrease if not at limit
			--index;
		}
	} else if (S2 == 1) {				// S1 pressed, increase voltage
		if (index < (NVOLTAGES-1)) {	// Increase if not at limit
			++index;
		}
	}
	P1IFG = 0;							// Clear interrupt
	LCDAVCTL1 = index << 1;				// LSB unused so shift left
	DisplayInt (voltages[index]);		// Display current voltage
}								// (0 means that the charge pump is off)
