// usi2cmastsm4.c - transmit temperature and receive delay, I2C master
// As usi2cmastsm2.c but uses USISTTIFG to start a new transaction
// 2 bytes, MSB first, new reading before transmission; Vref kept active
// Transactions every 0.7s triggered by watchdog as interval timer
// Simple state machine driven by USI interrupts, IFG and STTIFG
// F2013 on TI Experimenter's Board:
//   SCL on P1.6, SDA on P1.7, LED on P1.0 to show activity
// Calibrated 1MHz DCO, no crystal, ACLK from VLO
// J H Davies, 2008-06-09; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define LED		P1OUT_bit.P1OUT_0	// Output pin for LED (active high)
#define SLAVE_ADDRESS	0x46		// I2C address of remote slave
									// Address plus bit R/_W = 1
#define N_TRANSMIT	2				// Number of bytes to transmit
uint8_t TXdata[N_TRANSMIT] = {0,0};	// Data to transmit, initially 0
uint8_t delay = 0;					// Wait (delay+1) watchdog intervals
									//   between readings and transfers
void main (void)
{
	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
// Watchdog as interval timer, clear, ACLK / 8192 = 1.5Hz (0.7s)
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL | WDTIS0;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCL, SDA
	P1OUT = ~BIT0;					// LED low (off) others high
	P1DIR = BIT0;					// LED output, others input
	P1REN = ~BIT0;					// Pull unused pins up
// Configure SD16A: clock from SMCLK = 1MHz, internal reference on
	SD16CTL = SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 1024, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_1024 | SD16IE;
// PGA gain = 1, temperature sensor A6+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_6 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
// Enable SDA, SCL, master, hold module in reset
// Defaults: msb first, data output disabled, enable output latch
	USICTL0 = USIPE7 | USIPE6 | USIMST | USISWRST;
// I2C not SPI, interrupts on Start and counter, can't clear USIIFG
// Defaults: CPHA = 1 -> CKPH = 0, no interrupts on start
	USICTL1 = USII2C | USISTTIE | USIIE | USIIFG;
//	SCLK = SMCLK / 16, clock idles high (CPOL = CKPL = 1)
	USICKCTL = USIDIV_4 | USISSEL_2 | USICKPL;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	USICTL1 &= ~USIIFG;				// Clear flag, avoid unwanted intpt
	for (;;) {						// Loop forever with interrupts
		__low_power_mode_3();		// LPM3 between transactions
	}
}
//----------------------------------------------------------------------
// ISR for watchdog: activate SD16 to read temperature, change to LPM0
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	if (delay > 0) {				// Waited for enough intervals?
		--delay;					// No: decrement count
	} else {						// Yes: start conversion and TX
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion
// Change from LPM3 to LPM0 on exit to provide SMCLK for SD16 and USI
		__bic_SR_register_on_exit(LPM3_bits);
		__bis_SR_register_on_exit(LPM0_bits);
	}
}
//----------------------------------------------------------------------
// Wait for SD16A to complete conversion
//----------------------------------------------------------------------
// ISR for SD16A: compute temperature, start I2C
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Not acknowledged automatically
{
	uint32_t temperature;			// Converted value of temperature
									//   in units of 0.01degC
	temperature = SD16MEM0;			// Raw converted value, clears flag
	temperature *= 45454;			// Scaling factor (Vref/temp coeff)
	temperature += 0x8000;			// Allow for rounding in division
	temperature /= 0x10000;			// Divide by range of SD16A
	temperature -= 27300;			// Subtract offset to give celsius
	TXdata[0] = (uint8_t) temperature;		// Extract LSB and
	TXdata[1] = (uint8_t) (temperature >> 8);	//   MSB to transmit
	USICTL1 |= USISTTIFG;			// Set flag, request interrupt
//	Need to raise USIIFG as well or no interrupt is requested
	USICTL1 |= USIIFG;				// Set flag, request interrupt,
}									//   start I2C state machine
//----------------------------------------------------------------------
// ISR for USIIFG: State machine to control I2C slave transmission
//----------------------------------------------------------------------
#pragma vector = USI_VECTOR
__interrupt void USI_ISR (void)		// Not acknowledged automatically
{
	static uint8_t TXcount;			// Counter for bytes transmitted
	static enum {					// States of state machine
		Idle = 0,
		SendAddress = 2,
		ReceiveAddressAck = 4,
		CheckSendData = 6,
		ReceiveDataAck = 8,
		PrepareRestart = 10,
		ReSendAddress = 12,
		ReceiveReAddAck = 14,
		CheckGetData = 16,
		SendNack = 18,
		PrepareStop = 20,
		SendStop = 22		
	} I2CState = Idle;
	
	if (USICTL1 & USISTTIFG) {		// Start condition requested
		LED = 1;					// LED on while I2C active
// Don't clear STTIFG here - it becomes set again in SendAddresss
//		USICTL1 &= ~USISTTIFG;		// Acknowledge interrupt
		I2CState = SendAddress;		// Start state machine
	}
	switch (__even_in_range(I2CState, SendStop)) {
	case Idle:						// Should never happen!
		break;
	case SendAddress:				// Send Start, prepare address
		USISRL = 0;					// Low msb for Start condition
		USICTL0 |= USIOE;			// Enable output, grab SDA
		USICTL0 |= USIGE;			// Open output latch, drive 0 on SDA
// STTIFG is set (again) because a start condition appears on the bus
		USICTL0 &= ~USIGE;			// Hold 0 in output latch
		USISRL = SLAVE_ADDRESS << 1;	// Address with Write bit
		USICNT = USIFGCC | 8;		// Stop auto clr USIIFG, send 8 bits
		TXcount = N_TRANSMIT - 1;	// Initialize byte counter
// Must clear STTIFG here, not earlier
		USICTL1 &= ~USISTTIFG;		// Acknowledge interrupt
		I2CState = ReceiveAddressAck;	// Check address acknowledg next
		break;
	case ReceiveAddressAck:			// Receive acknowledgment after addr
		USICTL0 &= ~USIOE;			// Disable output, release SDA
		USICNT |= 1;				// Receive 1 bit
		I2CState = CheckSendData;	// Check Ack, send data byte next
		break;
	case CheckSendData:				// Send byte of data if last Ack'd
		if ((USISRL & BIT0) != 0) {	// Received Nack or Ack?
			USISRL = 0;				// Nack: abandon TX, prepare Stop
			USICTL0 |= USIOE;		// Enable output
			USICNT |= 1;			// Send 1 bit, pull SDA low
			I2CState = SendStop;	// Send Stop next
		} else {					// Ack: send byte of data
			USISRL = TXdata[TXcount];	// Load byte of data
			USICTL0 |= USIOE;		// Enable output
			USICNT |= 8;			// Send 8 bits
			I2CState = ReceiveDataAck;	// Receive ack next
		}
		break;
	case ReceiveDataAck:			// Receive acknowledgment after data
		USICTL0 &= ~USIOE;			// Disable output to receive Ack
		USICNT |= 1;				// Receive 1 bit
		if (TXcount == 0) {			// Was that the last byte?
			I2CState = PrepareRestart;	// Yes: send Restart next
		} else {
			--TXcount;					// No: Update byte counter
			I2CState = CheckSendData;	// Check Ack, send data next
		}
		break;

	case PrepareRestart:			// Prepare SDA = 1 for Start cond
		USISRL = 0xFF;				// Prepare pull SDA high for Start
		USICTL0 |= USIOE;			// Enable output
		USICNT |= 1;				// Send 1 bit, pull SDA high
		I2CState = ReSendAddress;	// Send address next
		break;
	case ReSendAddress:				// Send Restart and address to read
		USISRL = 0;					// Low msb for Start condition
		USICTL0 |= USIGE;			// Open output latch, drive 0 on SDA
		USICTL0 &= ~USIGE;			// Hold 0 in output latch
		USISRL = (SLAVE_ADDRESS << 1) | BIT0;	// Address with Read bit
		USICNT = 8;					// Send 8 bits
// Clear STTIFG again because it is set by Start that we have just sent
		USICTL1 &= ~USISTTIFG;		// Acknowledge interrupt
		I2CState = ReceiveReAddAck;	// Receive acknowledgment next
		break;
	case ReceiveReAddAck:			// Receive acknowledgment after addr
		USICTL0 &= ~USIOE;			// Disable output, release SDA
		USICNT |= 1;				// Receive 1 bit
		I2CState = CheckGetData;	// Check Ack, receive data byte next
		break;
	case CheckGetData:
		if ((USISRL & BIT0) != 0) {	// Received Nack or Ack?
			USISRL = 0;				// Nack: abandon RX, prepare Stop
			USICTL0 |= USIOE;		// Enable output
			USICNT |= 1;			// Send 1 bit, pull SDA low
			I2CState = SendStop;	// Send Stop next
		} else {					// Ack: receive byte of data
			USICNT |= 8;			// Receive 8 bits
			I2CState = SendNack;	// Send Nack next
		}
		break;
	case SendNack:
		delay = USISRL;				// Store received data
		USISRL = 0xFF;				// Send 1 for Nack
		USICTL0 |= USIOE;			// Enable output
		USICNT |= 1;				// Send 1 bit
		I2CState = PrepareStop;		// Finished: send Stop next
		break;
	case PrepareStop:				// Prepare to send Stop condition
		USISRL = 0;					// Prepare to pull SDA low for Stop
		USICTL0 |= USIOE;			// Enable output
		USICNT |= 1;				// Send 1 bit, pull SDA low
		I2CState = SendStop;		// Send Stop next
		break;
	case SendStop:
		USISRL = 0xFF;				// High msb for Stop condition
		USICTL0 |= USIGE;			// Open output latch, drive 1 on SDA
		USICTL0 &= ~USIGE;			// Hold 1 in output latch
		USICTL0 &= ~USIOE;			// Disable output to release SDA
		USICNT |= USISCLREL;		// Release hold on SCL
		I2CState = Idle;			// Return to idle state
		LED = 0;					// LED off when I2C inactive
// Change mode from LPM0 to LPM3 on exit: SMCLK no longer needed
		__bic_SR_register_on_exit(LPM0_bits);	// (not really necessary)
		__bis_SR_register_on_exit(LPM3_bits);
		break;
	}
	USICTL1 &= ~USIIFG;				// Clear flag, release SCL if held
}
