// usispiloop6.c - test SPI using USI with external loopback
// 8-bit transfer, SPI mode 3 (CPOL = CPHA = 1), every 0.7ms after WDT
// SDO and _SS driven high half a SCLK cycle after last bit received
// F2013 on TI Experimenter's Board:
//   SPI clock = SMCLK / 128 = 94kHz, so about 85us for transfer
//   SCLK on P1.5, MOSI (SDO) on P1.6, MISO (SDI) on P1.7
//   P1.4 used as _SS (nSS) but TIMING IS NOT PRECISE (done in software)
// Calibrated 12MHz DCO, no crystal, no ACLK
// J H Davies, 2007-11-02; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define nSS		P1OUT_bit.P1OUT_4	// Output pin for _SS (active low)

uint8_t RXdata, TXdata = 0x5A;		// Received and transmitted data,
									//   subsequent values incremented
void main (void)
{
	BCSCTL1 = CALBC1_12MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_12MHZ;			// Calibrated tap and modulation
// Watchdog as interval timer, clear, SMCLK / 8192 = 1.5kHz (0.7ms)
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTIS0;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCLK, SDI and SDO (but not nSS)
	P1OUT = BIT4;					// nSS high (inactive)
	P1DIR = BIT4;					// nSS output, others input
	P1REN = BIT0|BIT1|BIT2|BIT3;	// Pull resistors on unused pins
// Enable SDI, SDO, SCLK, msb first, master, enable output, latch data
	USICTL0 = USIPE7 | USIPE6 | USIPE5 | USIMST | USIOE | USISWRST;
// Write then read (CPHA = 1 -> CKPH = 0), SPI not I2C, enable interrupt
	USICTL1 = USIIE | USIIFG;		// Can't clear USIIFG in reset mode
//	SCLK = SMCLK / 128, clock idles high (CPOL = CKPL = 1)
	USICKCTL = USIDIV_7 | USISSEL_2 | USICKPL;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	USICTL1 &= ~USIIFG;				// Avoid unwanted interrupt
	for (;;) {						// Loop forever with interrupts
		__low_power_mode_0();		// LPM0 between interrupts
	}
}
//----------------------------------------------------------------------
// ISR for watchdog: activate nSS, start new SPI transfer
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	nSS = 0;						// Lower nSS (make active)
	USISRL = TXdata;				// Load shift register for transfer
	USICNT = 8;						// Start SPI to transfer 8 bits
}
//----------------------------------------------------------------------
// ISR for USI: acknowledge, store received data, wait 1/2 SCLK cycle,
//   drive SDO high by toggling USIGE, deactivate nSS
// Delay = 64 MCLK cycles less (6 interrupt + 3 + 6 + 1 + 4 + 4) = 40
//----------------------------------------------------------------------
#define DELAY_LOOPS	12				// assume 3 MCLK cycles per loop
#pragma vector = USI_VECTOR
__interrupt void USI_ISR (void)		// NOT acknowledged automatically
{
	volatile uint16_t i;			// Delay loop counter

	USICTL1 &= ~USIIFG;				// Clear flag (no bitfield?)
	RXdata = USISRL;				// Store received data (low byte)
	TXdata = RXdata + 1;			// Slightly different next time!
	USISR = 0xFFFF;					// Ensure that a 1 will be output
	for (i = DELAY_LOOPS; i > 0; --i) {
	}								// Wait for about half SCLK cycle
	USICTL0 |= USIGE;				// Make latch transparent, output 1
	USICTL0 &= ~USIGE;				// Lock latch, output remains 1
	nSS = 1;						// Raise nSS (make inactive)
}
